@@ -47,7 +47,8 @@ struct ksmbd_startup_request {
 	__u32	share_fake_fscaps;
 	__u32	sub_auth[3];
 	__u32	smb2_max_credits;
-	__u32   reserved[128];		/* Reserved room */
+	__u32	smbd_max_io_size;	/* smbd read write size */
+	__u32   reserved[127];		/* Reserved room */
 	__u32	ifc_list_sz;
 	__s8	____payload[];
 };
