@@ -554,14 +554,6 @@ static void process_group_kv(gpointer _k, gpointer _v, gpointer user_data)
 			clear_share_flag(share,	KSMBD_SHARE_FLAG_INHERIT_OWNER);
 	}
 
-	if (shm_share_config(k, KSMBD_SHARE_CONF_FOLLOW_SYMLINKS)) {
-		if (cp_get_group_kv_bool(v))
-			set_share_flag(share, KSMBD_SHARE_FLAG_FOLLOW_SYMLINKS);
-		else
-			clear_share_flag(share,
-				KSMBD_SHARE_FLAG_FOLLOW_SYMLINKS);
-	}
-
 	if (shm_share_config(k, KSMBD_SHARE_CONF_VFS_OBJECTS)) {
 		char *p;
 		int i;
